import os
import sys
from abc import ABC, abstractmethod
from typing import Dict, List

from terminaltables import AsciiTable

from digipathos import Dataset, DataLoader


class Command(ABC):
    def __init__(self,
                 desc: str,
                 command,
                 lang: str,
                 require_args: bool = False,
                 args_description: str = None,
                 arg_type=str):
        self.desc = desc
        self.command = command
        self.require_args = require_args
        self.args_description = args_description
        self.arg_type = arg_type
        self.lang = lang

        if args_description:
            self.require_args = True

    def execute(self, *args):
        if self.require_args:
            casted_arg = self.arg_type(args[0])
            self.print_pretty(self.command(casted_arg))
        else:
            self.print_pretty(self.command())

        input('Press Enter to return to the main menu')
        # cls()

    @abstractmethod
    def print_pretty(self, command_response):
        raise NotImplementedError("You should override the method print_pretty")


def cls():
    os.system('cls' if os.name == 'nt' else 'clear')


class ListDatasets(Command):
    def print_pretty(self, command_response: Dict[str, Dataset]) -> None:
        dataset_ids = ['ID']
        dataset_crops = ['Crop']
        dataset_disorders = ['Disorder']

        for dataset_id, dataset in command_response.items():
            dataset_ids.append(dataset_id)
            dataset_crops.append(dataset.get_crop_name(lang=self.lang))
            dataset_disorders.append(dataset.get_disorder_name(lang=self.lang))

        output = [dataset_ids, dataset_crops, dataset_disorders]
        transpose_and_print(output)


class ListCrops(Command):
    def print_pretty(self, command_response: Dict[str, List[int]]) -> None:
        datasets_crops = ['Crop']
        for name in command_response.keys():
            datasets_crops.append(name)

        output = [datasets_crops]
        transpose_and_print(output)


class ListDatasetsFromCrop(Command):
    def print_pretty(self, command_response: List[Dataset]) -> None:
        dataset_ids = ['ID']
        dataset_disorders = ['Disorder']

        for dataset in command_response:
            dataset_ids.append(dataset.get_id())
            dataset_disorders.append(dataset.get_disorder_name(lang=self.lang))

        output = [dataset_ids, dataset_disorders]
        transpose_and_print(output)


class DownloadDataset(Command):
    def print_pretty(self, command_response):
        pass


class DownloadAllDatasets(Command):
    def print_pretty(self, command_response):
        pass


class DownloadAllDatasetsFromCrop(Command):
    def print_pretty(self, command_response):
        pass


class Exit(Command):
    def print_pretty(self, command_response):
        pass


def transpose_and_print(data: List) -> None:
    output = list(map(list, zip(*data)))
    print(AsciiTable(output).table)


def get_commands(data_loader: DataLoader) -> Dict[int, Command]:
    commands = {
        0: Exit(desc='Exit the application',
                command=sys.exit,
                lang=data_loader.lang),

        1: ListDatasets(desc='List all datasets',
                        command=data_loader.get_datasets,
                        lang=data_loader.lang),

        2: ListCrops(desc='List all crops',
                     command=data_loader.get_crops,
                     lang=data_loader.lang),

        3: ListDatasetsFromCrop(desc='List all datasets from a given crop',
                                command=data_loader.get_datasets_from_crop,
                                require_args=True,
                                args_description='Crop name',
                                lang=data_loader.lang),

        4: DownloadDataset(desc='Download single dataset',
                           command=data_loader.download_dataset,
                           require_args=True,
                           args_description='Dataset id',
                           arg_type=int,
                           lang=data_loader.lang),

        5: DownloadAllDatasets(desc='Download all datasets',
                               command=data_loader.download_all_datasets,
                               lang=data_loader.lang),

        6: DownloadAllDatasetsFromCrop(desc='Download all datasets from a given crop',
                                       command=data_loader.download_datasets_from_crop,
                                       arg_type=str,
                                       args_description='Crop name',
                                       lang=data_loader.lang)
    }

    return commands



from abc import ABC, abstractmethod
from typing import List, Dict

import requests

from digipathos import Dataset
from digipathos.constants import ENDPOINT_ITEMS, BASE_URL

class DigipathosRepository(ABC):
    def __init__(self, lang: str = 'pt'):
        self.__items = {}
        self.__crops = {}
        self.lang = lang

    @abstractmethod
    def load_datasets(self) -> List[Dataset]:
        raise NotImplementedError("You should override the method get_items")

    def load_crops(self) -> Dict[str, Dataset]:
        if not self.__items:
            self.__items = self.load_datasets()

        crops = {}

        for item_id, dataset in self.__items.items():
            crop_name = dataset.get_crop_name(lang=self.lang)
            if crop_name not in crops:
                crops[crop_name] = [dataset]
            else:
                crops[crop_name].append(dataset)

        self.__crops = crops

        return crops

    def load_items_from_crop(self, crop_name: str):
        if not self.__items:
            self.__items = self.load_datasets()

        if not self.__crops:
            self.__crops = self.load_crops()

        if crop_name not in self.__crops:
            raise ValueError("Crop name %s is not present on crop list" % crop_name)

        return self.__crops[crop_name]

    def load_item(self, item_id: int) -> Dataset:
        if not self.__items:
            self.__items = self.load_datasets()

        return self.__items[item_id]


class DigipathosRemoteApi(DigipathosRepository):
    def load_datasets(self) -> List[Dataset]:
        url = BASE_URL + ENDPOINT_ITEMS
        resp = requests.get(url=url)
        data = resp.json()

        json_items = data['bitstreams']
        items = {}

        for item in json_items:
            item = Dataset.make_from_json_dict(item)
            item_id = item.get_id()

            items[item_id] = item

        self.__items = items

        return items


class DigipathosMockedApi(DigipathosRepository):
    def load_datasets(self):
        pass



import re

from digipathos.constants import BASE_URL

VALID_LANGUAGES = ['pt', 'en']
pattern_en = '\(([^)]+)\)'
pattern_pt = '(?<!\S)\w+(?!\S)'


class Dataset:
    """
    Used to store all the info from a given dataset
    """

    def __init__(self,
                 full_name: str,
                 url: str,
                 extension: str,
                 size: float):
        self.full_name = full_name
        self.extension = extension
        self.size = size

        self.url = BASE_URL + url
        self.id = self.get_id()

    @classmethod
    def make_from_json_dict(cls, item: dict):
        """
        Used to create a Dataset object from a json response
        :param item: the json item dict
        :return: a new Dataset object
        """
        return cls(full_name=item['name'],
                   url=item['bsLink'],
                   extension=item['format'],
                   size=item['size'])

    def validate_language(self, lang: str):
        """
        Used to validate if the passed lang argument is valid or not. In case it isn't, it will throw a ValueError
        exception
        :param lang: the lang to be validated
        :return: ValueError if language is not valid
        """
        if lang not in VALID_LANGUAGES:
            raise ValueError('You should provide a valid language. Valid languages: %s' % VALID_LANGUAGES)

    def get_crop_name(self, lang: str = 'en'):
        """
        Used to get the crop name from the dataset for a specified language
        :param lang: the language to retrieve the crop name
        :return: the crop name
        """
        self.validate_language(lang)

        if lang == 'en':
            name = self.find(pattern_en, self.full_name, 0)
        elif lang == 'pt':
            name = self.find(pattern_pt, self.full_name, 0)

        return name

    def get_disorder_name(self, lang: str = 'en'):
        """
        Used to get the disorder name from the dataset for a specified language
        :param lang: the language to retrieve the disorder name
        :return: the disorder name
       """
        self.validate_language(lang)

        if lang == 'en':
            name = self.find(pattern_en, self.full_name, 1)
        elif lang == 'pt':
            name = self.find(pattern_pt, self.full_name, 1)

        return name

    def find(self, pattern: str, contents: str, index: int) -> str:
        """
        Used to find a text in a given index based on a RegEx pattern
        :param pattern: the pattern to be searched
        :param contents: the text to be used
        :param index: the index to be fetched
        :return: the located text in the given index that matches the pattern
        """
        try:
            return re.findall(pattern, contents)[index]
        except IndexError:
            return None

    def get_id(self) -> int:
        """
        Used to get the id from a given dataset.
        :return: the id (int)
        """
        item_id = self.url.split('/')[6]
        return int(item_id)



import os
import urllib

from digipathos import Dataset


def download_dataset(dataset: Dataset, output_dir: str, verbose=True) -> None:
    """
    This method is used to download a given dataset into the disk.
    :param dataset: The dataset object to be persisted into the disk
    :param output_dir: Where the file should be stored
    """
    if not os.path.exists(output_dir):
        os.makedirs(output_dir, exist_ok=True)

    file_path = os.path.join(output_dir, "%s.%s" % (dataset.id, dataset.extension.lower()))

    if verbose:
        print('Starting download.. This may take a while to finish. Your download will be ready at: %s ' % file_path)

    urllib.request.urlretrieve(dataset.url, file_path)



import unittest
from unittest.mock import patch

from digipathos.cli import browser


class CliTests(unittest.TestCase):

    @patch("builtins.input", return_value="invalid_option")
    def test_invalid_command_throws_exception(self, mock_input):
        with self.assertRaises(Exception) as e:
            browser.read_command()
            self.assertEqual(e.message, "Invalid command")


if __name__ == "__main__":
    unittest.main()



import os
from typing import List, Dict

from digipathos import Dataset
from digipathos.data import DigipathosRemoteApi
from digipathos.utils import download_utils


class DataLoader:
    """
    This class should be used to load data from the Digipathos API
    """

    def __init__(self,
                 artifacts_path: str = 'artifacts',
                 lang: str = 'en',
                 auto_fetch=True):
        """
        DataLoader basic initializer
        :param artifacts_path: where the artifacts will be stored after downloaded
        :param lang: the language to be used to fetch names (crops and disorders)
        :param auto_fetch: should data be fetched automatically on initialization
        :param repository: the repository to be used. You can replace it here for another DataSource, but it must be a
        child of DigitpathosRepository
        """
        self.artifacts_path = artifacts_path
        self.lang = lang
        self.repository = DigipathosRemoteApi(self.lang)

        if auto_fetch:
            self.items = self.repository.load_datasets()
            self.crops = self.repository.load_crops()

    def get_datasets(self) -> List[Dataset]:
        """
        Used to get all the datasets without downloading them. In case you are looking
        to download, please refer to the download_all_datasets method.
        :return: a List containing all the available datasets
        """
        return self.repository.load_datasets()

    def get_crops(self) -> Dict[str, Dataset]:
        """
        Used to get all the crops without downloading them
        :return: a dictionary containing all the datasets
        """
        return self.repository.load_crops()

    def get_datasets_from_crop(self, crop_name: str) -> List[Dataset]:
        """
        Used to get the list of all datasets related to a given crop without downloading them. In case you are looking
        to download, please refer to the download_datasets_from_crop method.
        :param crop_name: the name of the crop
        :return: a List containing all the Datasets from a given crop
        """
        return self.repository.load_items_from_crop(crop_name)

    def get_dataset(self, dataset_id: int) -> Dataset:
        """
        Used to get the information about a given dataset without downloading it. In case you are looking to download,
        please refer to the download_dataset method.
        :param dataset_id: the dataset id
        :return: a Dataset object containing the dataset
        """
        return self.repository.load_item(dataset_id)

    def download_dataset(self, dataset_id: int) -> None:
        """
        Used to download a given dataset to the disk
        :param dataset_id: the id of the dataset
        """
        item = self.repository.load_item(dataset_id)
        download_utils.download_dataset(item, self.artifacts_path)

    def download_all_datasets(self) -> None:
        """
        Used to download all the datasets
        """
        datasets = self.repository.load_datasets()

        for dataset in datasets:
            download_utils.download_dataset(dataset, self.artifacts_path)

    def download_datasets_from_crop(self, crop_name: str) -> None:
        """
        Used to download all the datasets from a given crop name. This method will output the download crops into a new
        folder with the same name as the crop. e.g.: /artifacts/<crop_name>
        :param crop_name: the name of the crop
        """
        items = self.repository.load_items_from_crop(crop_name)

        output_path = os.path.join(self.artifacts_path, crop_name)

        for item in items:
            download_utils.download_dataset(item, output_path)
